/*
 * Copyright © 2018-2022 Koherence, LLC (www.koherence.com).
 *
 * This file is part of the Entangle ATSC 3.0 Base Stack.
 *
 * The Entangle ATSC 3.0 Base Stack is free software: you can redistribute it
 * and/or modify it under the terms of the GNU Affero General Public License
 * version 3 as published by the Free Software Foundation with the addition of
 * the following permission added to Section 15 as permitted in Section 7(a):
 * FOR ANY PART OF THE COVERED WORK IN WHICH THE COPYRIGHT IS OWNED BY
 * KOHERENCE, LLC, KOHERENCE, LLC DISCLAIMS THE WARRANTY OF NON INFRINGEMENT
 * OF THIRD PARTY RIGHTS.
 *
 * The Entangle ATSC 3.0 Base Stack is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA.
 *
 * You can contact Koherence, LLC at email address info@koherence.com.
 */

#ifndef ENATSC3_SPECDEFS_H
#define ENATSC3_SPECDEFS_H

// This file contains structures and values corresponding to the A/3*
// specifications.

// ALP packet type, from A/330 5.2
typedef enum
{
    ENATSC3ALPPacketType_IPV4 = 0,
    ENATSC3ALPPacketType_CompressedIP = 2,
    ENATSC3ALPPacketType_LinkLayerSignaling = 4,
    ENATSC3ALPPacketType_PacketTypeExtension = 6,
    ENATSC3ALPPacketType_MPEG2TS = 7,
} ENATSC3ALPPacketType;

typedef enum
{
    ENATSC3ALPSignalingType_LinkMappingTable = 1,
    ENATSC3ALPSignalingType_ROHCUDescriptionTable = 2
} ENATSC3ALPSignalingType;

typedef enum
{
    ENATSC3ALPSignalingFormat_Binary = 0,
    ENATSC3ALPSignalingFormat_XML = 1,
    ENATSC3ALPSignalingFormat_JSON = 2
} ENATSC3ALPSignalingFormat;

typedef enum
{
    ENATSC3ALPSignalingEncoding_Uncompressed = 0,
    ENATSC3ALPSignalingEncoding_DEFLATE = 1
} ENATSC3ALPSignalingEncoding;

typedef struct
{
    uint32_t signaling_type;
    uint32_t signaling_type_extension;
    uint32_t signaling_version;
    uint32_t signaling_format;
    uint32_t signaling_encoding;
} __attribute__((__packed__)) ENATSC3ALPLinkLayerSignalingHeader;

// The following is the four-byte header for LLS tables. See A/331 table 6.1
typedef struct
{
    uint8_t lls_table_id;
    uint8_t lls_group_id;
    uint8_t group_count_minus1;
    uint8_t lls_table_version;
} __attribute__((__packed__)) ENATSC3LLSTableHeader;

typedef struct
{
    // The precision of the L1 time associated with the packet.
    // 0: L1 time is not present
    // 1: millisecond precision
    // 2: microsecond precision
    // 3: nanosecond precision.
    uint8_t precision;

    // The L1 time (valid only if fL1TimePresent is true). The time is broken into
    // four parts: the seconds component, the milliseconds component, the microseconds
    // component, and the nanoseconds component. For example for time 1536829.123456789,
    // the seconds component hold 1536829, the milliseconds component 123, the
    // microseconds component 456, and the nanoseconds component 789.
    uint32_t seconds;
    uint32_t milliseconds;
    uint32_t microseconds;
    uint32_t nanoseconds;
} ENATCS3L1Time;

typedef enum 
{
    ENATSC3LCTChannelObjectFormatId_Reserved = 0,
    ENATSC3LCTChannelObjectFormatId_File = 1,
    ENATSC3LCTChannelObjectFormatId_Entity = 2,
    ENATSC3LCTChannelObjectFormatId_UnsignedPackage = 3,
    ENATSC3LCTChannelObjectFormatId_SignedPackage = 4,
} ENATSC3LCTChannelObjectFormatId;

typedef enum
{
    ENATSC3ROUTECodepoint_Reserved = 0,
    ENATSC3ROUTECodepoint_NRTFileMode = 1,
    ENATSC3ROUTECodepoint_NRTEntityMode = 2,
    ENATSC3ROUTECodepoint_NRTUnsignedPackageMode = 3,
    ENATSC3ROUTECodepoint_NRTSignedPackageMode = 4,
    ENATSC3ROUTECodepoint_NewISTimelineChanged = 5,
    ENATSC3ROUTECodepoint_NewISTimelineContinued = 6,
    ENATSC3ROUTECodepoint_RedundantIS = 7,
    ENATSC3ROUTECodepoint_MediaSegmentFileMode = 8,
    ENATSC3ROUTECodepoint_MediaSegmentEntityMode = 9,
} ENATSC3ROUTECodepoint;

#endif
