/*
 * Copyright © 2018-2023 Koherence, LLC (www.koherence.com). 
 *
 * This file is part of the Entangle ATSC 3.0 Base Stack.
 *
 * The Entangle ATSC 3.0 Base Stack is free software: you can redistribute it
 * and/or modify it under the terms of the GNU Affero General Public License
 * version 3 as published by the Free Software Foundation with the addition of 
 * the following permission added to Section 15 as permitted in Section 7(a): 
 * FOR ANY PART OF THE COVERED WORK IN WHICH THE COPYRIGHT IS OWNED BY 
 * KOHERENCE, LLC, KOHERENCE, LLC DISCLAIMS THE WARRANTY OF NON INFRINGEMENT 
 * OF THIRD PARTY RIGHTS.
 *
 * The Entangle ATSC 3.0 Base Stack is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write 
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, 
 * Boston, MA 02110-1301 USA.
 *
 * You can contact Koherence, LLC at email address info@koherence.com
 */

#ifndef ENATSC3CAP_H
#define ENATSC3CAP_H

#define ENATSC3CAP_HEADER_MAGIC 0x656e

// The format version indicates the capture file structure.
typedef enum
{
    ENATSC3CapFormat_V2 = 2,
} ENATSC3CapFormat;

// The format type indicates the contents of the file
typedef enum
{
    ENATSC3CapType_Invalid = 0,

    // Each record contains an integral number of MPEG2-TS packets
    ENATSC3CapType_ATSC1_MPEGTS = 1,

    // Each record contains one baseband packet or one demodulator packet
    ENATSC3CapType_ATSC3_Baseband = 2,

    // Each record contains one ALP packet or one demodulator packet
    ENATSC3CapType_ATSC3_ALP = 3,

    // Each record contains an integral number of J.288 MPEG2-TS packets
    // which encapsulate ALP packets
    ENATSC3CapType_ATSC3_J288ALP = 4,
} ENATSC3CapType;

typedef enum
{
    ENATSC3CapPayloadType_Undefined = 0,
    ENATSC3CapPayloadType_ATSC1_MPEGTS = 1,
    ENATSC3CapPayloadType_ATSC3_Baseband = 2,
    ENATSC3CapPayloadType_ATSC3_ALP = 3,
    ENATSC3CapPayloadType_SCTE_MPEGTS = 4,
    ENATSC3CapPayloadType_ATSC3_J288ALP = 5,

    // Record type 254 contains demodulator information as structured in
    // ENATSC3CapDemodulatorRecordV2. It is found in V2 files.
    ENATSC3CapPayloadType_Demod_V2 = 254,
} ENATSC3CapPayloadType;

// The modcod for a PLP
typedef enum
{
    ENATSC3CapModulation_Unknown = 0,
    ENATSC3CapModulation_QPSK = 1,
    ENATSC3CapModulation_16QAM = 2,
    ENATSC3CapModulation_64QAM = 3,
    ENATSC3CapModulation_256QAM = 4,
    ENATSC3CapModulation_1024QAM = 5,
    ENATSC3CapModulation_4096QAM = 6,
} ENATSC3CapModulation;

typedef enum
{
    ENATSC3CapCoding_Unknown = 0,
    ENATSC3CapCoding_2_15 = 2,
    ENATSC3CapCoding_3_15 = 3,
    ENATSC3CapCoding_4_15 = 4,
    ENATSC3CapCoding_5_15 = 5,
    ENATSC3CapCoding_6_15 = 6,
    ENATSC3CapCoding_7_15 = 7,
    ENATSC3CapCoding_8_15 = 8,
    ENATSC3CapCoding_9_15 = 9,
    ENATSC3CapCoding_10_15 = 10,
    ENATSC3CapCoding_11_15 = 11,
    ENATSC3CapCoding_12_15 = 12,
    ENATSC3CapCoding_13_15 = 13,
} ENATSC3CapCoding;

// The file header for V2 files
typedef struct
{
    // One of ENATSC3CapFormat
    uint8_t format;

    // One of ENATSC3CapType
    uint8_t type;

    // The magic value has value 0x656e ('en') and is used to distinguish 
    // between headerless (v0 or v1) and headered file formats. In v0 and v1
    // files this corresponds to reserved fields and should be 0.
    uint16_t magic;
} __attribute__((__packed__)) ENATSC3CapFileHeader;

typedef struct
{
    uint32_t L1B_version;
    uint32_t L1B_mimo_scattered_pilot_encoding;
    uint32_t L1B_lls_flag;
    uint32_t L1B_time_info_flag;
    uint32_t L1B_return_channel_flag;
    uint32_t L1B_papr_reduction;
    uint32_t L1B_frame_length_mode;
    uint32_t L1B_frame_length;
    uint32_t L1B_excess_samples_per_symbol;
    uint32_t L1B_time_offset;
    uint32_t L1B_additional_samples;
    uint32_t L1B_num_subframes;
    uint32_t L1B_preamble_num_symbols;
    uint32_t L1B_preamble_reduced_carriers;
    uint32_t L1B_L1_Detail_content_tag;
    uint32_t L1B_L1_Detail_size_bytes;
    uint32_t L1B_L1_Detail_fec_type;
    uint32_t L1B_L1_Detail_additional_parity_mode;
    uint32_t L1B_L1_Detail_total_cells;
    uint32_t L1B_first_sub_mimo;
    uint32_t L1B_first_sub_miso;
    uint32_t L1B_first_sub_fft_size;
    uint32_t L1B_first_sub_reduced_carriers;
    uint32_t L1B_first_sub_guard_interval;
    uint32_t L1B_first_sub_num_ofdm_symbols;
    uint32_t L1B_first_sub_scattered_pilot_pattern;
    uint32_t L1B_first_sub_scattered_pilot_boost;
    uint32_t L1B_first_sub_sbs_first;
    uint32_t L1B_first_sub_sbs_last;
    uint32_t L1B_reserved1;
    uint16_t L1B_reserved2;
    uint32_t L1B_crc;
} __attribute__((__packed__)) ATSC3_L1_Basic_signaling;

typedef struct
{
    uint8_t L1D_version;
    uint8_t L1D_num_rf;
    uint16_t vL1D_bonded_bsid[8];
    uint32_t L1D_time_sec;
    uint16_t L1D_time_msec;
    uint16_t L1D_time_usec;
    uint16_t L1D_time_nsec;

    uint16_t L1D_bsid;
    uint32_t L1D_crc;

    // L1B_num_subframes is pulled from L1B and indicates the number of 
    // _ATSC1_L1_Detail_Subframe structures that follow.
    uint8_t L1B_num_subframes;

    struct _ATSC3_L1_Detail_Subframe
    {
        uint8_t L1D_mimo;
        uint8_t L1D_miso;
        uint8_t L1D_fft_size;
        uint8_t L1D_reduced_carriers;
        uint8_t L1D_guard_interval;
        uint16_t L1D_num_ofdm_symbols;
        uint8_t L1D_scattered_pilot_pattern;
        uint8_t L1D_scattered_pilot_boost;
        uint8_t L1D_sbs_first;
        uint8_t L1D_sbs_last;

        uint8_t L1D_subframe_multiplex;
        uint8_t L1D_frequency_interleaver;
        uint16_t L1D_sbs_null_cells;
        uint8_t L1D_num_plp;
        struct _ATSC3_L1_Detail_signaling_subframe_plp
        {
            uint8_t L1D_plp_id;
            uint8_t L1D_plp_lls_flag;
            uint8_t L1D_plp_layer;
            uint32_t L1D_plp_start;
            uint32_t L1D_plp_size;
            uint8_t L1D_plp_scrambler_type;
            uint8_t L1D_plp_fec_type;
            uint8_t L1D_plp_mod;
            uint8_t L1D_plp_cod;
            uint8_t L1D_plp_TI_mode;
            uint32_t L1D_plp_fec_block_start;
            uint32_t L1D_plp_CTI_fec_block_start;
            uint8_t L1D_plp_num_channel_bonded;
            uint8_t L1D_plp_channel_bonding_format;
            uint8_t vL1D_plp_bonded_rf_id[8];
            uint8_t L1D_plp_mimo_stream_combining;
            uint8_t L1D_plp_mimo_IQ_interleaving;
            uint8_t L1D_plp_mimo_PH;
            uint8_t L1D_plp_type;
            uint16_t L1D_plp_num_subslices;
            uint32_t L1D_plp_subslice_interval;
            uint8_t L1D_plp_TI_extended_interleaving;
            uint8_t L1D_plp_CTI_depth;
            uint16_t L1D_plp_CTI_start_row;
            uint8_t L1D_plp_HTI_inter_subframe;
            uint8_t L1D_plp_HTI_num_ti_blocks;
            uint16_t L1D_plp_HTI_num_fec_blocks_max;
            uint16_t L1D_plp_HTI_num_fec_blocks;
            uint16_t vL1D_plp_HTI_num_fec_blocks[17];
            uint8_t L1D_plp_HTI_cell_interleaver;
            uint8_t L1D_plp_ldm_injection_level;
        } __attribute__((__packed__)) vPLPs[64];
    } __attribute__((__packed__)) vSubframes[0];
} __attribute__((__packed__)) ATSC3_L1_Detail_signaling;

typedef struct
{
    // The frequency the tuner was tuned to 
    uint32_t frequencyHz;

    // Whether the tuner reported locked to a signal of the standard. For
    // ATSC3 this implies that the demodulator detected an ATSC 3 emission
    // and not that any PLPs have locked. See per-plp locking below.
    uint8_t lock;

    // SNR * 100
    int32_t snr;

    // RSSI * 100
    uint32_t rssi;

    // BER (bits/s). For ATSC3 this is summed across PLPs.
    uint32_t ber;

    // Packet error count (since start of capture)
    uint32_t packetErrorCount;
   
    union
    {
        struct 
        {
            // Bitmask of PLPs which are present in the emission. Extracted
            // from L1D for ease of processing.
            uint64_t plpPresentMask;

            // Bitmask of PLPs carrying LLS. Extracted from L1D for ease of
            // processing.
            uint64_t llsPresentMask;

            // Bitmask of PLPs enabled for demodulation. Information for PLPs
            // not enabled may be invalid depending on the demodulator.
            uint64_t plpEnabledMask;

            // Lock status of each PLP
            uint64_t plpLockedMask;

            // per-plp signal quality metrics
            struct
            {
                // Signal quality (0 - 100)
                uint32_t snq;

                // BER * 100
                uint32_t ber;
            } vSignalQuality[64];

            ATSC3_L1_Basic_signaling l1_basic_signaling;

            // NOTE that ATSC3_L1_Detail_signaling is variable-sized
            // and depends on the number of subframes.
            ATSC3_L1_Detail_signaling l1_detail_signaling;
        }  __attribute__((__packed__)) atsc3;
     } __attribute__((__packed__));
} __attribute__((__packed__)) ENATSC3CapDemodulatorRecordV2;

// The capture format used for V2 files
typedef struct
{
    // The type of data carried in this packet, one of ENATSC3CapPayloadType:
    // 1: ATSC 1.0 MPEG2-TS packets
    // 2: ATSC 3.0 Baseband packet
    // 3: ATSC 3.0 ALP Packet 
    // 4: SCTE MPEG-TS
    // 254: V2 Demodulator record
    // 255: V0 (headerless) Demodulator record
    uint8_t type;

    // Counter of captured records
    uint8_t continuityCounter;

    // Error flag - 0 if the packet(s) contained in this record are error-free,
    // 1 otherwise.
    uint8_t fError;

    // For ATSC 3.0, the plp from which the ALP or baseband packet originated. 
    // For J288-ALP captures, this field is only populated if a single PLP
    // was captured. Otherwise it is set to 255 and the PLP should have been
    // communicated via extension headers in the ALP packet. 
    // Not used for ATSC 1.0 or demodulator status.
    uint8_t plpId;

    // The number of bytes of payload FOLLOWING the header.
    uint32_t length;

    // The value of CLOCK_MONOTONIC_RAW when the packet(s) were made available
    // to the capture application.
    struct
    {
        uint64_t sec;
        uint32_t nsec;
    } __attribute__((__packed__)) monowhen;

    // The value of CLOCK_REALTIME when the packet(s) were made available
    // to the capture application.
    struct
    {
        uint64_t sec;
        uint32_t nsec;
    } __attribute__((__packed__)) rtwhen;

    // ATSC 3.0, L1 time information. Not used for ATSC 1.0 or demod status.
    //
    // NOTE: The RZR driver caches this information and there may be a race 
    // condition between the packet being made available to the capture 
    // application and the driver updating the cached L1 information. 
    struct
    {
        uint8_t reserved0;
        uint8_t reserved1;
        uint8_t reserved2;

        // A322 9.4: 
        // 0: Time information is not included
        // 1: Time information is signaled to ms precision
        // 2: Time information is signaled to us precision
        // 3: Time information is signaled to ns precision
        uint8_t L1B_time_info_flag;

        // Raw L1 values
        uint32_t L1D_time_sec;
        uint32_t L1D_time_msec;
        uint32_t L1D_time_usec;
        uint32_t L1D_time_nsec;
    } __attribute__((__packed__)) broadcastTime;
} __attribute__((__packed__)) ENATSC3CapRecordHeaderV2;

#endif
