/*
 * Copyright © 2020-2022 Koherence, LLC (www.koherence.com).
 *
 * This file is part of the Entangle ATSC 3.0 Base Stack.
 *
 * The Entangle ATSC 3.0 Base Stack is free software: you can redistribute it
 * and/or modify it under the terms of the GNU Affero General Public License
 * version 3 as published by the Free Software Foundation with the addition of 
 * the following permission added to Section 15 as permitted in Section 7(a): 
 * FOR ANY PART OF THE COVERED WORK IN WHICH THE COPYRIGHT IS OWNED BY 
 * KOHERENCE, LLC, KOHERENCE, LLC DISCLAIMS THE WARRANTY OF NON INFRINGEMENT 
 * OF THIRD PARTY RIGHTS.
 *
 * The Entangle ATSC 3.0 Base Stack is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write 
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, 
 * Boston, MA 02110-1301 USA.
 *
 * You can contact Koherence, LLC at email address info@koherence.com
 */

#ifndef LIBENATSC3J288ALP_H
#define LIBENATSC3J288ALP_H

typedef struct
{
    // See A/322 9.2.1. The time_info_flag indicates whether the physical
    // layer frame from which the L1B was extracted contains timing
    // information, and if so to what precision. Valid values are:
    // 0: Time information is not included in the current frame.
    // 1: Time information is included in the current frame and signaled
    //    to ms precision.
    // 2: Time information is included in the current frame and signaled
    //    to us precision.
    // 3: Time information is included in the current frame and signaled
    //    to ns precision.
    int32_t L1B_time_info_flag;

    // See A/322 9.3.1.
    uint32_t L1D_time_sec;
    uint16_t L1D_time_msec;
    uint16_t L1D_time_usec;
    uint16_t L1D_time_nsec;
} ENATSC3J288PTPInfo;

typedef struct _ENATSC3J288ALPHandle_t *ENATSC3J288ALPHandle_t;

typedef void(*ENATSC3J288BasebandCallbackFunction)(uint32_t plpId, 
    struct timespec *pPacketTimeUTC, uint64_t packetNumber, 
    int32_t fError, int32_t fDiscontinuity, const uint8_t *pPacket, 
    int32_t sPacket, void *pUserData);

typedef void(*ENATSC3J288ALPCallbackFunction)(uint32_t plpId, 
    struct timespec *pPacketTimeUTC, uint64_t packetNumber, 
    int32_t fError, int32_t fDiscontinuity, const uint8_t *pPacket, 
    int32_t sPacket, void *pUserData);

typedef void(*ENATSC3J288PTPCallbackFunction)(
    struct timespec *pPacketTimeUTC, uint64_t packetNumber, 
    ENATSC3J288PTPInfo *pPTPInfo, void *pUserData);

ENStatus
ENATSC3J288ALPOpen(ENATSC3J288ALPHandle_t *phATSC3J288ALP);

ENStatus
ENATSC3J288ALPClose(ENATSC3J288ALPHandle_t hATSC3J288ALP);

ENStatus
ENATSC3J288ALPRegisterCallbacks(ENATSC3J288ALPHandle_t hATS3J288ALP,
    ENATSC3J288ALPCallbackFunction pALPCallback,
    ENATSC3J288BasebandCallbackFunction pBasebandCallback, 
    ENATSC3J288PTPCallbackFunction pPTPCallback,
    void *pUserData);

ENStatus
ENATSC3J288ALPProcessJ288Packets(ENATSC3J288ALPHandle_t hATSC3J288ALP,
    struct timespec *pPacketTimeUTC, const uint8_t *pData, int32_t nBytes);

ENStatus
ENATSC3J288ALPSetVerbosity(ENATSC3J288ALPHandle_t hATSC3J288ALP, 
    ENVerbosity verbosity);

#endif
