/*
 * Copyright © 2018-2022 Koherence, LLC (www.koherence.com). 
 *
 * This file is part of the Entangle ATSC 3.0 Base Stack.
 *
 * The Entangle ATSC 3.0 Base Stack is free software: you can redistribute it
 * and/or modify it under the terms of the GNU Affero General Public License
 * version 3 as published by the Free Software Foundation with the addition of 
 * the following permission added to Section 15 as permitted in Section 7(a): 
 * FOR ANY PART OF THE COVERED WORK IN WHICH THE COPYRIGHT IS OWNED BY 
 * KOHERENCE, LLC, KOHERENCE, LLC DISCLAIMS THE WARRANTY OF NON INFRINGEMENT 
 * OF THIRD PARTY RIGHTS.
 *
 * The Entangle ATSC 3.0 Base Stack is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write 
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, 
 * Boston, MA 02110-1301 USA.
 *
 * You can contact Koherence, LLC at email address info@koherence.com
 */

#ifndef ATSC3LLS_AEAT_H
#define ATSC3LLS_AEAT_H

// See A/331 6.5

typedef struct
{
    EnXMLParserNodeBase base;

    int32_t eventCodeDataOffset;        // A code identifying the event type of
                                        // the AEA message
    int32_t typeDataOffset;             // A national-assigned string 
                                        // designating the domain of the code
} ATSC3LLSAEAT_EventCodeNode;

#define ATSC3LLSAEATEventCodeNodeAttribute_eventCode            0x0001
#define ATSC3LLSAEATEventCodeNodeAttribute_type                 0x0002

typedef struct
{
    EnXMLParserNodeBase base;

    int32_t eventDescDataOffset;        // The short pain text description of
                                        // the emergency event
    int32_t langDataOffset;             // The code denoting the language of
                                        // the respective element of the 
                                        // EventDesc
} ATSC3LLSAEAT_EventDescNode;

#define ATSC3LLSAEATEventDescNodeAttribute_eventDesc            0x0001
#define ATSC3LLSAEATEventDescNodeAttribute_lang                 0x0002

typedef struct
{
    EnXMLParserNodeBase base;

    int32_t locationDataOffset;         // The geographic code delineating the
                                        // affected area of the AEA message.
    int32_t typeDataOffset;             // A national-assigned string 
                                        // designating the domain of the code
} ATSC3LLSAEAT_LocationNode;

#define ATSC3LLSAEATLocationNodeAttribute_location              0x0001
#define ATSC3LLSAEATLocationNodeAttribute_type                  0x0002

typedef struct
{
    EnXMLParserNodeBase base;

    int64_t effectiveMillisecondsUTC;   // The effective time of the AEA
                                        // message. It appears when the 
                                        // @aeaType is "alert" or "update". If
                                        // omitted, the default is immediate.
    int64_t expiresMillisecondsUTC;     // The expiration time of the AEA
                                        // message. It apears when the
                                        // @aeaType is "alert" or "update".
    ATSC3LLSAEAT_EventCodeNode eventCode;   // A code identifying the event 
                                        // type of the AEA message.
    EnXMLParserDataOffsetArray eventDescs; // Array of
                                        // ATSC3LLSAEAT_EventDescNode
                                        // The short plain text 
                                        // description of the emergency event 
                                        // (e.g. "Tornado Warning" or "Tsunami 
                                        // Warning".
    EnXMLParserDataOffsetArray locations; // Array of ATSC3LLSAEAT_LocationNode 
                                        // The geographic code delineating
                                        // the affected area of the AEA
                                        // message. It appears when the
                                        // @aeaType is "alert" or "update" and
                                        // can appear when the @aeaType is 
                                        // "cancel".
} ATSC3LLSAEAT_HeaderNode;

#define ATSC3LLSAEATHeaderNodeAttribute_effective               0x0001
#define ATSC3LLSAEATHeaderNodeAttribute_expires                 0x0002
#define ATSC3LLSAEATHeaderChildNode_EventCode                   0x0004
#define ATSC3LLSAEATHeaderChildNode_EventDesc                   0x0008
#define ATSC3LLSAEATHeaderChildNode_Location                    0x0010

typedef struct 
{
    EnXMLParserNodeBase base;

    int32_t aeaTextDataOffset;          // Contains the specific text of the
                                        // emergency notification. 
    int32_t langDataOffset;             // The code denoting the language of
                                        // the respective element of the AEA 
                                        // message text.
} ATSC3LLSAEAT_AEATextNode;

#define ATSC3LLSAEATAEATextNodeAttribute_text                   0x0001
#define ATSC3LLSAEATAEATextNodeAttribute_lang                   0x0002

typedef struct
{
    EnXMLParserNodeBase base;

    int32_t serviceNameDataOffset;    // A user-friendly name for the service 
                                        // where the LiveMedia is available.
    int32_t langDataOffset;           // The language of the text described
                                        // in the ServiceName element.
} ATSC3LLSAEAT_ServiceNameNode;

#define ATSC3LLSAEATServiceNameNodeAttribute_serviceName        0x0001
#define ATSC3LLSAEATServiceNameNodeAttribute_lang               0x0002

typedef struct 
{
    EnXMLParserNodeBase base;

    uint32_t vBSIDs[16];                // Identifier of the Broadcast Stream
                                        // containing the emergency-related 
                                        // A/V service.
    int32_t nBSIDs;                     // number of BSIDs.
    uint32_t serviceId;                 // Integer number that identifies the
                                        // emergency-related A/V service.
    EnXMLParserDataOffsetArray serviceNames; // Array of 
                                        // ATSC3LLSAEAT_ServiceNameNode
                                        // A user-friendly name for the
                                        // serice where the LiveMedia is 
                                        // available
} ATSC3LLSAEAT_LiveMediaNode;

#define ATSC3LLSAEATLiveMediaNodeAttribute_bsid                 0x0001
#define ATSC3LLSAEATLiveMediaNodeAttribute_serviceId            0x0002
#define ATSC3LLSAEATLiveMediaChildNode_ServiceName              0x0004

typedef struct
{
    EnXMLParserNodeBase base;

    int32_t langDataOffset;             // The code denoting the language of 
                                        // the respective element Media.
    int32_t mediaDescDataOffset;        // Text describing the type and content 
                                        // of the media file.
    int32_t mediaTypeDataOffset;        // Text identifying the intended use of 
                                        // the associated media.
    int32_t urlDataOffset;              // URL of the media file.
    int32_t alternateUrlDataOffset;     // Alternate URL of the media file when
                                        // it is also available via non-
                                        // broadcast delivery (i.e. via the
                                        // internet).
    int32_t contentTypeDataOffset;      // IANA media type of media content
                                        // referenced by @url.
    uint32_t contentLength;             // Size in bytes of media content
                                        // referenced by @url.
    int32_t mediaAssocDataOffset;       // URI of another Media element with
                                        // which this attribute is associated.
} ATSC3LLSAEAT_MediaNode;

#define ATSC3LLSAEATMediaNodeAttribute_lang                     0x0001
#define ATSC3LLSAEATMediaNodeAttribute_mediaDesc                0x0002
#define ATSC3LLSAEATMediaNodeAttribute_mediaType                0x0004
#define ATSC3LLSAEATMediaNodeAttribute_url                      0x0008
#define ATSC3LLSAEATMediaNodeAttribute_alternateUrl             0x0010
#define ATSC3LLSAEATMediaNodeAttribute_contentType              0x0020
#define ATSC3LLSAEATMediaNodeAttribute_contentLength            0x0040
#define ATSC3LLSAEATMediaNodeAttribute_mediaAssoc               0x0080

typedef struct
{
    EnXMLParserNodeBase base;

    int32_t aeaIdDataOffset;            // The identifier of AEA message
    int32_t issuerDataOffset;           // The identifier of the broadcast
                                        // station originating or forwarding
                                        // the message.
    int32_t audienceDataOffset;         // The intended distribution of the 
                                        // AEA message.
    int32_t aeaTypeDataOffset;          // Thte category of the message
    int32_t refAEAIdDataOffset;         // The referenced identifier of AEA
                                        // message. It shall appear when the 
                                        // @aeaType is "alert" or "update" 
                                        // and may appear when the @aeaType is
                                        // "cancel".
    uint32_t priority;                  // The priority of the message. It 
                                        // shall appear whenthe @aeaType is 
                                        // "alert" or "update" ad may appear
                                        // when the @aeaType is "cancel".
    int32_t wakeup;                     // Indication that this AEA is 
                                        // associated with a wake-up event.
    ATSC3LLSAEAT_HeaderNode header;     // The container for the basic alert
                                        // envelope.
    EnXMLParserDataOffsetArray aeaTexts; // Array of ATSC3LLSAEAT_AEATextNode
                                        // Contains the specific text of the
                                        // emergency notification. It appears
                                        // when the @aeaType is "alert" or 
                                        // "update" and can appear when the
                                        // @aeaType is "cancel".
    ATSC3LLSAEAT_LiveMediaNode liveMedia; // Contains the information of 
                                        // emergency-related live A/V
                                        // service which is delivered in the
                                        // broadcast stream.
    EnXMLParserDataOffsetArray medias;  // Array of 
                                        // ATSC3LLSAEAT_MediaNode
} ATSC3LLSAEAT_AEANode;

#define ATSC3LLSAEATAEANodeAttribute_aeaId                      0x0001
#define ATSC3LLSAEATAEANodeAttribute_issuer                     0x0002
#define ATSC3LLSAEATAEANodeAttribute_audience                   0x0004
#define ATSC3LLSAEATAEANodeAttribute_aeaType                    0x0008
#define ATSC3LLSAEATAEANodeAttribute_refAEAId                   0x0010
#define ATSC3LLSAEATAEANodeAttribute_priority                   0x0020
#define ATSC3LLSAEATAEANodeAttribute_wakeup                     0x0040
#define ATSC3LLSAEATAEAChildNode_Header                         0x0080
#define ATSC3LLSAEATAEAChildNode_AEAText                        0x0100
#define ATSC3LLSAEATAEAChildNode_LiveMedia                      0x0200
#define ATSC3LLSAEATAEAChildNode_Media                          0x0400

typedef struct
{
    EnXMLParserNodeBase base;

    int32_t aeaTableIdDataOffset;       // The AEAT Table Id

    EnXMLParserDataOffsetArray aeas;    // Array of
                                        // ATSC3LLSAEAT_AEANode
    EnXMLParserDataBuffer dataBuffer;
} ATSC3LLSAEAT_AEATNode;

#define ATSC3LLSAEATNodeAttribute_aeaTableId                    0x0001
#define ATSC3LLSAEATChildNode_AEA                               0x0002

typedef enum
{
    ATSC3LLSAEATNodeType_Invalid = 0,
    ATSC3LLSAEATNodeType_AEAT = 1,
    ATSC3LLSAEATNodeType_AEA = 2,
    ATSC3LLSAEATNodeType_Header = 3,
    ATSC3LLSAEATNodeType_EventCode = 4,
    ATSC3LLSAEATNodeType_EventDesc = 5,
    ATSC3LLSAEATNodeType_Location = 6,
    ATSC3LLSAEATNodeType_AEAText = 7,
    ATSC3LLSAEATNodeType_LiveMedia = 8,
    ATSC3LLSAEATNodeType_ServiceName = 9,
    ATSC3LLSAEATNodeType_Media = 10
} ATSC3LLSAEATNodeType;

ENStatus
ATSC3LLSParseAEAT(const char *pXML, int32_t sXML, 
    ATSC3LLSAEAT_AEATNode *pParsedATSC3LLSAEAT);

ENStatus
ATSC3LLSFreeParsedAEAT(ATSC3LLSAEAT_AEATNode *pParsedATSC3LLSAEAT);

#endif
