/*
 * Copyright © 2019-2022 Koherence, LLC (www.koherence.com). 
 *
 * This file is part of the Entangle ATSC 3.0 Base Stack.
 *
 * The Entangle ATSC 3.0 Base Stack is free software: you can redistribute it
 * and/or modify it under the terms of the GNU Affero General Public License
 * version 3 as published by the Free Software Foundation with the addition of 
 * the following permission added to Section 15 as permitted in Section 7(a): 
 * FOR ANY PART OF THE COVERED WORK IN WHICH THE COPYRIGHT IS OWNED BY 
 * KOHERENCE, LLC, KOHERENCE, LLC DISCLAIMS THE WARRANTY OF NON INFRINGEMENT 
 * OF THIRD PARTY RIGHTS.
 *
 * The Entangle ATSC 3.0 Base Stack is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write 
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, 
 * Boston, MA 02110-1301 USA.
 *
 * You can contact Koherence, LLC at email address info@koherence.com
 */

#ifndef ENATSC3_ATSC3SLS_HELD_H
#define ENATSC3_ATSC3SLS_HELD_H

typedef struct
{
    EnXMLParserNodeBase base;

    int32_t appContextIdDataOffset;
    int32_t requiredCapabilitiesDataOffset;
    int32_t appRendering;
    int64_t clearAppContextCacheDateSecondsUTC;
    int32_t bcastEntryPackageUrlDataOffset;
    int32_t bcastEntryPageUrlDataOffset;
    int32_t bbandEntryPageUrlDataOffset;
    int64_t validFromSecondsUTC;
    int64_t validUntilSecondsUTC;
    int32_t coupledServicesDataOffset; 
    int32_t lctTSIRefDataOffset;
} ENATSC3SLSHELDHTMLEntryPackageNode;

#define ENATSC3SLSHELDHTMLEntryPackage_appContextId              0x0001
#define ENATSC3SLSHELDHTMLEntryPackage_requiredCapabilities      0x0002
#define ENATSC3SLSHELDHTMLEntryPackage_appRendering              0x0004
#define ENATSC3SLSHELDHTMLEntryPackage_clearAppContextCacheDate  0x0008
#define ENATSC3SLSHELDHTMLEntryPackage_bcastEntryPackageUrl      0x0010
#define ENATSC3SLSHELDHTMLEntryPackage_bcastEntryPageUrl         0x0020
#define ENATSC3SLSHELDHTMLEntryPackage_bbandEntryPageUrl         0x0040
#define ENATSC3SLSHELDHTMLEntryPackage_validFrom                 0x0080
#define ENATSC3SLSHELDHTMLEntryPackage_validUntil                0x0100
#define ENATSC3SLSHELDHTMLEntryPackage_coupledServices           0x0200
#define ENATSC3SLSHELDHTMLEntryPackage_lctTSIRef                 0x0400

typedef struct
{
    EnXMLParserNodeBase base;

    EnXMLParserDataOffsetArray htmlEntryPackages;

    EnXMLParserDataBuffer dataBuffer;
} ENATSC3SLSHELDNode;

typedef enum
{
    ENATSC3SLSHELDNodeType_Invalid = 0,
    ENATSC3SLSHELDNodeType_HELD = 1,
    ENATSC3SLSHELDNodeType_HTMLEntryPackage = 2,
} ENATSC3SLSHELDNodeType;

ENStatus
ENATSC3SLSHELDParse(const char *pXML, int32_t sXML,
    ENATSC3SLSHELDNode *pParsedHELD);

ENStatus
ENATSC3SLSHELDFree(ENATSC3SLSHELDNode *pParsedHELD);

#endif
