/*
 * Copyright © 2020-2022 Koherence, LLC (www.koherence.com).
 *
 * This file is part of the Entangle ATSC 3.0 Base Stack.
 *
 * The Entangle ATSC 3.0 Base Stack is free software: you can redistribute it
 * and/or modify it under the terms of the GNU Affero General Public License
 * version 3 as published by the Free Software Foundation with the addition of 
 * the following permission added to Section 15 as permitted in Section 7(a): 
 * FOR ANY PART OF THE COVERED WORK IN WHICH THE COPYRIGHT IS OWNED BY 
 * KOHERENCE, LLC, KOHERENCE, LLC DISCLAIMS THE WARRANTY OF NON INFRINGEMENT 
 * OF THIRD PARTY RIGHTS.
 *
 * The Entangle ATSC 3.0 Base Stack is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write 
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, 
 * Boston, MA 02110-1301 USA.
 *
 * You can contact Koherence, LLC at email address info@koherence.com
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <stddef.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <signal.h>
#include <errno.h>
#include <dirent.h>
#include <libgen.h>
#include <inttypes.h>
#include <pthread.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <arpa/inet.h>

#include <libendebug/libendebug.h>

#include "enatsc3bitratestatistics.h"

static char *ensys = "utils";
static char *ensubsys = "bstats";

#define NANOSECONDS_PER_SECOND 1000000000
#define MILLISECONDS_PER_SECOND 1000
#define NANOSECONDS_PER_MILLISECOND (NANOSECONDS_PER_SECOND / MILLISECONDS_PER_SECOND)

void
ENATSC3BitrateStatisticsInitialize(ENATSC3BitrateStatistics *pStatistics, 
    int32_t reportPeriodMillieconds)
{
    memset(pStatistics, 0, sizeof(*pStatistics));
    pStatistics->reportPeriodMillieconds = reportPeriodMillieconds;
}

void
ENATSC3BitrateStatisticsUpdate(ENATSC3BitrateStatistics *pStatistics,
    struct timespec *pTPacket, int64_t sPacket)
{
    if (pStatistics->tStatPeriodStart.tv_sec == 0)
    {
        pStatistics->tStatPeriodStart = *pTPacket;
        pStatistics->nBytes = 0;
    }
    else
    {
        int64_t duration;

        pStatistics->nBytes += sPacket;

        duration = pTPacket->tv_sec - pStatistics->tStatPeriodStart.tv_sec;
        duration *= 1000;
        duration += (pTPacket->tv_nsec - pStatistics->tStatPeriodStart.tv_nsec) /
            NANOSECONDS_PER_MILLISECOND;

        if (duration >= pStatistics->reportPeriodMillieconds)
        {
            LOGPRINT_NOFILE(
                "%ld.%06ld: %24s: %d bits/s (%d bytes %"PRId64" ms)\n",
                pTPacket->tv_sec, pTPacket->tv_nsec / 1000,
                pStatistics->strName,
                (pStatistics->nBytes * 8) * 1000 / duration,
                pStatistics->nBytes, duration);
            pStatistics->tStatPeriodStart = *pTPacket;
            pStatistics->nBytes = 0;
        }
    }
}

