/*
 * Copyright © 2020-2022 Koherence, LLC (www.koherence.com).
 *
 * This file is part of the Entangle ATSC 3.0 Base Stack.
 *
 * The Entangle ATSC 3.0 Base Stack is free software: you can redistribute it
 * and/or modify it under the terms of the GNU Affero General Public License
 * version 3 as published by the Free Software Foundation with the addition of 
 * the following permission added to Section 15 as permitted in Section 7(a): 
 * FOR ANY PART OF THE COVERED WORK IN WHICH THE COPYRIGHT IS OWNED BY 
 * KOHERENCE, LLC, KOHERENCE, LLC DISCLAIMS THE WARRANTY OF NON INFRINGEMENT 
 * OF THIRD PARTY RIGHTS.
 *
 * The Entangle ATSC 3.0 Base Stack is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write 
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, 
 * Boston, MA 02110-1301 USA.
 *
 * You can contact Koherence, LLC at email address info@koherence.com
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <stddef.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <signal.h>
#include <errno.h>
#include <dirent.h>
#include <libgen.h>
#include <inttypes.h>
#include <pthread.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <arpa/inet.h>

#include <libendebug/libendebug.h>
#include <enatsc3-common/enatsc3.h>

#include "enatsc3strings.h"

char *ensys = "utils";
char *ensubsys = "strings";

char *
ENATSC3MediaTypeToString(ENATSC3MediaType mediaType)
{
    switch (mediaType)
    {
        case ENATSC3MediaType_Audio:
            return "audio";
        case ENATSC3MediaType_Video:
            return "video";
        case ENATSC3MediaType_Subtitle:
            return "ttml";
        case ENATSC3MediaType_Invalid:
            return "invalid";
        default:
            break;
    }

    return "unknown";
}

char *
ENATSC3CodecToString(ENATSC3Codec codec)
{
    switch (codec)
    {
        case ENATSC3Codec_Invalid:
            return "invalid";
        case ENATSC3Codec_HEV1:
            return "hev1";
        case ENATSC3Codec_HVC1:
            return "hvc1";
        case ENATSC3Codec_AAC:
            return "aac";
        case ENATSC3Codec_AC4:
            return "ac4";
        case ENATSC3Codec_TTML:
            return "ttml";
    }

    return "unknown";
}

char *
ENATSC3SLSProtocolToString(ENATSC3SLSProtocol slsProtocol)
{
    switch (slsProtocol)
    {
        case ENATSC3SLSProtocol_Reserved:
            return "reserved";
        case ENATSC3SLSProtocol_ROUTE:
            return "route";
        case ENATSC3SLSProtocol_MMTP:
            return "mmtp";
    }

    return "unknown";
}
