/*
 * Copyright © 2018-2022 Koherence, LLC (www.koherence.com).
 *
 * This file is part of the Entangle ATSC 3.0 Base Stack.
 *
 * The Entangle ATSC 3.0 Base Stack is free software: you can redistribute it
 * and/or modify it under the terms of the GNU Affero General Public License
 * version 3 as published by the Free Software Foundation with the addition of
 * the following permission added to Section 15 as permitted in Section 7(a):
 * FOR ANY PART OF THE COVERED WORK IN WHICH THE COPYRIGHT IS OWNED BY
 * KOHERENCE, LLC, KOHERENCE, LLC DISCLAIMS THE WARRANTY OF NON INFRINGEMENT
 * OF THIRD PARTY RIGHTS.
 *
 * The Entangle ATSC 3.0 Base Stack is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA.
 *
 * You can contact Koherence, LLC at email address info@koherence.com
 */

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <unistd.h>
#include <ctype.h>
#include <signal.h>
#include <pthread.h>
#include <assert.h>

#include <libendebug/libendebug.h>

#include "libenutils_defs.h"
#include "libenutils.h"

static char *ensys = "enutils";
static char *ensubsys = "crash";

#define ColorDefault    "\x1B[0m"
#define ColorRed        "\x1B[31m"

void
ENUtils_CrashAssertInternal(char *pFilename, int line, char *pReason, int32_t fImmediate)
{
    struct timespec utcnow;

    clock_gettime(CLOCK_REALTIME, &utcnow);

    fprintf(stderr, 
        ColorRed "%ld.%06ld  FORCING CRASH AT %s:%d: %s\n" ColorDefault, 
        utcnow.tv_sec, utcnow.tv_nsec / 1000, pFilename, line, pReason);
    ERRORPRINT("ASSERTION FAILED AT %s:%d: %s\n", pFilename, line, pReason);
    if (!fImmediate)
    {
        EnLogFlush();
    }
    assert(0);
}

void
ENUtils_DebugAssertInternal(char *pFilename, int line, char *pReason, int32_t fImmediate)
{
    struct timespec utcnow;

    clock_gettime(CLOCK_REALTIME, &utcnow);

    ERRORPRINT("DEBUG ASSERTION FAILED AT %s:%d: %s\n", pFilename, line, pReason);
    if (!fImmediate)
    {
        EnLogFlush();
    }
#ifdef DEBUG
    assert(0);
#endif
}

void
ENUtils_CrashHandler(int signum)
{
    struct sigaction act;

    // Uninstall our handlers
    act.sa_handler = SIG_DFL;
    sigemptyset(&act.sa_mask);
    act.sa_flags = 0;
    sigaction(SIGSEGV, &act, NULL);

    act.sa_handler = SIG_DFL;
    sigemptyset(&act.sa_mask);
    act.sa_flags = 0;
    sigaction(SIGBUS, &act, NULL);

    EnLogFlush(); 
}

void
ENUtils_InstallSEGVHandler(void)
{
    struct sigaction act;

    act.sa_handler = ENUtils_CrashHandler;
    sigemptyset(&act.sa_mask);
    act.sa_flags = 0;
    sigaction(SIGSEGV, &act, NULL);

    act.sa_handler = ENUtils_CrashHandler;
    sigemptyset(&act.sa_mask);
    act.sa_flags = 0;
    sigaction(SIGBUS, &act, NULL);
}
